import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:hangman/config/default_game_settings.dart';

class GameSettings {
  final String gameMode;
  final String gameLevel;

  GameSettings({
    required this.gameMode,
    required this.gameLevel,
  });

  static String getGameModeValueFromUnsafe(String gameMode) {
    if (DefaultGameSettings.allowedGameModeValues.contains(gameMode)) {
      return gameMode;
    }

    return DefaultGameSettings.defaultGameModeValue;
  }

  static String getGameLevelValueFromUnsafe(String gameLevel) {
    if (DefaultGameSettings.allowedGameLevelValues.contains(gameLevel)) {
      return gameLevel;
    }

    return DefaultGameSettings.defaultGameLevelValue;
  }

  factory GameSettings.createDefault() {
    return GameSettings(
      gameMode: DefaultGameSettings.defaultGameModeValue,
      gameLevel: DefaultGameSettings.defaultGameLevelValue,
    );
  }

  void dump() {
    printlog('$GameSettings:');
    printlog('  ${DefaultGameSettings.parameterCodeGameMode}: $gameMode');
    printlog('  ${DefaultGameSettings.parameterCodeGameLevel}: $gameLevel');
    printlog('');
  }

  @override
  String toString() {
    return '$GameSettings(${toJson()})';
  }

  Map<String, dynamic>? toJson() {
    return <String, dynamic>{
      DefaultGameSettings.parameterCodeGameMode: gameMode,
      DefaultGameSettings.parameterCodeGameLevel: gameLevel,
    };
  }
}
